#include <Servo.h>

Servo allServos[4];                // create servo objects to control a servo
String serialString;              // incoming serial string
String servoPositionSubstring;    // we use this to store the substrings we extract from the incoming serial string
int servoPositions[4];            // store positions of the servos in this array of integers
// how much serial data we expect before a newline
const unsigned int MAX_INPUT = 20;

void setup() {
  for (int i = 0; i < 4; i++) {
    allServos[i].attach(5 + i);  // asign the servoes to a pin starting at 5
    allServos[i].write(0);       // set the servos to thier zero position
  }

  Serial.begin(57600);          // setup the serial oject with a baud rate
}

void loop() {
  while (Serial.available () > 0) {
    processIncomingByte (Serial.read ());
  }
}


void processIncomingByte (const byte inByte)
{
  static char input_line [MAX_INPUT];
  static unsigned int input_pos = 0;

  switch (inByte)
  {

    case '\n':   // end of text
      input_line [input_pos] = 0;  // terminating null byte

      // terminator reached! process input_line here ...
      process_data (input_line);

      // reset buffer for next time
      input_pos = 0;
      break;

    case '\r':   // discard carriage return
      break;

    default:
      // keep adding if not full ... allow for terminating null byte
      if (input_pos < (MAX_INPUT - 1))
        input_line [input_pos++] = inByte;
      break;

  }  // end of switch

} 

void process_data (const char * data)
{
  serialString = data;

  // the serial messages are sent as a string with the format A001B001C001D001| we use substring with indexOf to separate the servo positions we want from the rest of the string and then convert them to integers

  servoPositionSubstring = serialString.substring(serialString.indexOf('A') + 1, serialString.indexOf('B') );
  servoPositions[0] = servoPositionSubstring.toInt();

  servoPositionSubstring = serialString.substring(serialString.indexOf('B') + 1, serialString.indexOf('C') );
  servoPositions[1] = servoPositionSubstring.toInt();

  servoPositionSubstring = serialString.substring(serialString.indexOf('C') + 1, serialString.indexOf('D') );
  servoPositions[2] = servoPositionSubstring.toInt();

  servoPositionSubstring = serialString.substring(serialString.indexOf('D') + 1, serialString.indexOf('|') );
  servoPositions[3] = servoPositionSubstring.toInt();

  for (int i = 0; i < 4; i++) {
    allServos[i].write(servoPositions[i]);
  }
}
